function SelectMultipleController($scope, $rootScope, templateManager, ScreenService, ZhUtil, SelectService, ZHPromise, metaDataFactory, EventChildPropagation) {
	var self = this;

	function overrideSelectWidgetSelectAllFunction(field){
		var unSelectAllOriginalFunction = field.selectWidget.unSelectAll;
		field.selectWidget.unSelectAll = function(){
			unSelectAllOriginalFunction();
			field.selectWidget.updateGroups();
		};

		var selectAllOriginalFunction = field.selectWidget.selectAll;
		field.selectWidget.selectAll = function(rows){
			selectAllOriginalFunction(rows);
			field.selectWidget.updateGroups();
		};
	}

	function buildWidgetActions(field){
		var actions = [
			{
				label: ScreenService.i18n('Cancel'),
				showAsAction: 'back',
				isVisible: true,
				showOnForm: true,
				showOnList: true,
				hideIcon: true,
				events: [{
					name: 'ActionEvent',
					code: 'args.owner.widget.selectField.cancelSelection(); args.owner.widget.afterClosePopup(args.owner.widget.selectField);',
					id: 9998
				}]
			},
			{
				label: ScreenService.i18n('Ok'),
				showAsAction: 'never',
				isVisible: true,
				showOnForm: true,
				showOnList: true,
				events: [{
					name: 'ActionEvent',
					code: 'args.owner.widget.setSelected(args.owner.widget.dataSource.getCheckedRows(), args.owner.widget.selectField); args.owner.widget.afterClosePopup(args.owner.widget.selectField);',
					id: 9999
				}]
			}
		];

		if (field.showAddButtonAs == 'action') {
			actions.push({
				icon: 'plus',
				label: field.addButtonText,
				showAsAction: 'always|edit|checked_rows|view',
				isVisible: true,
				showOnForm: true,
				showOnList: true,
				events: [{
					name: 'ActionEvent',
					code: 'args.owner.widget.onAdd()',
					id: 9999
				}]
			});
		}
		return actions;
	}

	function buildWidgetTemplate(field){
		var template;
		if (!field.showSelectAs) {
			template = "widget/grid.html";
		} else if (field.showSelectAs === "tree") {
			template = "widget/tree/grid.html";
		} else {
			template = "widget/" + field.showSelectAs + ".html";
		}
		return template;
	}

	function buildWidgetFields(field){
		var fields;
		if (field.fields && field.fields.length) {
			fields = field.fields;
		} else {
			fields = [
				{
					label: field.sourceLabel || field.label,
					name: field.descriptionField,
					translate: field.translate,
					showOnList: true
				}
			];
		}
		return fields;
	}

	function buildWidgetEvents(field){
		var events = [{
			name: "WidgetAfterMoveRow",
			code: "args.owner.toggle(args.row, args.owner);",
			id: 9999
		},
		{
			name: "WidgetAfterLoadDataSource",
			code: "args.owner.afterLoadSelectDataSource(args.owner);",
			id: 10000
		},
		{
			name: "WidgetAfterSelectRow",
			code: "args.owner.updateGroups();",
			id: 10001
		},
		{
			name: "WidgetAfterUnselectRow",
			code: "args.owner.updateGroups();",
			id: 10002
		}];
		return EventChildPropagation.propagate(events, field);
	}

	function buildWidgetFloatingControl(field){
		var floatingControl = {
			openSearch: !Util.isMobile(),
			defaultSearchField: field.defaultSearchField || field.descriptionField,
			pdfAction: false,
			xlsAction: false,
			csvAction: false,
			searchAction: true,
			customizationAction: {
				filter: false,
				layout: false,
				view: false
			}
		};
		return floatingControl;
	}

	function copyFieldProperties(field, obj){
		var copiedObj = obj || {};
		var mappedPropertiesToCopy = {
			styles: 'selectWidgetStyles'
		};
		var propertiesToCopy = [
			'parentProperty',
			'primaryProperty',
			'expandingProperty',
			'breadcrumbProperty',
			'selectCascade',
			'childrenDependency',
			'reverse',
			'bringGroupsCollapsed',
			'onAdd',
			'showAddButtonAs',
			'addButtonText',
			'noMatchText',
			'searchNoMatchText',
			'itemsPerPage',
			'onDemand',
			'searchCriteria',
			'selectAllWithoutRequest',
			'dataSource',
			'dataSourceFilter',
		];

		Object.keys(mappedPropertiesToCopy).forEach(function(propertyName){
			var mappedPropertyName = mappedPropertiesToCopy[propertyName];
			copiedObj[propertyName] = field[mappedPropertyName];
		});
		propertiesToCopy.forEach(function(propertyName){
			copiedObj[propertyName] = field[propertyName];
		});
		return copiedObj;
	}

	function buildSelectWidget(field){
		var selectWidget = {
			id: 9999,
			name: 'select_multiple_widget',
			label: ScreenService.i18n(field.label),
			container: field.widget.container,
			isVisible: true,
			popupNoBlock: false,
			showCheckbox: true,
			showDescriptionOrder: false,
			autoFocus: false,
			focusable: true,
			
			order: field.order,
			fields: buildWidgetFields(field),
			events: buildWidgetEvents(field),
			actions: buildWidgetActions(field),
			template: buildWidgetTemplate(field),
			floatingControl: buildWidgetFloatingControl(field),

			selectField: field,
			searchWithPagination: Util.isUndefinedOrNull(field.searchWithPagination) || field.searchWithPagination,
			hasPagination: Util.isUndefinedOrNull(field.hasPagination) || field.hasPagination,
		};
		return copyFieldProperties(field, selectWidget);
	}

	function _openField(field) {
		var defer = ZHPromise.defer();
		var updateCheckedRows = function() {
			if (field.buildOptionsFromDataSource) {
				field.buildOptionsFromDataSource(field.value());
			}

			if (Util.isArray(field.widget.currentRow[field.name]) && !field.dataSource.allToggled) {
				var rowsToRemove = [];
				if (Util.isArray(field.valueFields)) {
					rowsToRemove = field.dataSource.getCheckedRows().filter(function(row) {
						return field.widget.currentRow[field.name].every(function(value) {
							return field.valueFields.some(function(key) {
								return value[key] !== row[key];
							});
						});
					});
				} else {
					rowsToRemove = field.dataSource.getCheckedRows().filter(function(row) {
						return field.widget.currentRow[field.name].indexOf(row[field.valueField]) === -1;
					});
				}
				field.selectWidget.dataSource.removeCheckedRows(rowsToRemove);

			} else if(!field.dataSource.allToggled){
				field.selectWidget.dataSource.clearCheckedRows();
			}
			defer.resolve();
		};

		innerSearch("", field);

		field.allInitialState = field.dataSource.allToggled;

		field.cancelSelection = function(){
			field.dataSource.allToggled = field.allInitialState;
			ScreenService.closePopup();
		};

		var isToReload = !!(field.selectWidget && Util.objectHasAnyValue(field.selectWidget.searchCriteria));
		if (!field.selectWidget) {
			var selectWidget = buildSelectWidget(field);
			field.selectWidget = metaDataFactory.widgetFactory(selectWidget);

			field.selectWidget.setSelected = setSelected;
			field.selectWidget.afterClosePopup = afterClosePopup;
			field.selectWidget.afterChangeOrder = afterChangeOrder;
			field.selectWidget.afterLoadSelectDataSource = afterLoadSelectDataSource;
			field.selectWidget.setAllToggledInDataSource = setAllToggledInDataSource;

			isToReload = !!(field.dataSource && field.dataSource.lazyLoad);
			overrideSelectWidgetSelectAllFunction(field);
		}

		if (isToReload) {
			field.selectWidget.searchCriteria = {};
			field.selectWidget.searchList = "";
			field.reload().then(updateCheckedRows, updateCheckedRows);
		} else {
			updateCheckedRows();
		}

		ScreenService.openPopup(field.selectWidget);
		return defer.promise;
	}

	$scope.openField = function(field) {
		var e = false;
		if (!field.isReadOnly()) {
			if (field.beforeSelectOpen) {
				e = field.beforeSelectOpen();
				if (e.isDefaultPrevented()) {
					return;
				}
			}
			// remove focus from element
			field.element.blur();
			window.getSelection().removeAllRanges(); //bug webkit

			_openField(field).then(function() {
				if (field.afterSelectOpen) {
					field.afterSelectOpen();
				}
			});
		}
	};

	function innerSearch(selectListSearch, field) {
		if (Util.isArray(field.filterDefinitions) && field.filterDefinitions.length) {
			var dataSource = field.dataSource;
			var filter = ZhUtil.buildFiltersInField(field, selectListSearch);
			dataSource.filter(filter, function(data) {
				dataSource.data = data;
				templateManager.updateTemplate();
			});
		}
	}

	var setChanged = function(field) {
		field.widget.currentRow.changed = true;
	};

	function setAllToggledInDataSource(){
		if(this.dataSource.allToggled){
			var rowsToCheck = [];
			var dataSource = this.dataSource;
			dataSource.updateCheckedRows();
			if(!dataSource.uncheckedRows.length){
				rowsToCheck = this.dataSource.data;
			} else {
				rowsToCheck = this.dataSource.data.filter(function(row) {
					 var index = dataSource.uncheckedRows.some(function(element){
						 return angular.equals(element, row);
					 });
					 return index;
				 });
			}
			dataSource.addCheckedRows(rowsToCheck);
			dataSource.removeUncheckedRows();
		}
	}

	function setSelected(rows, field) {
		ScreenService.closePopup();

		if(field.dataSource.allToggled){
			changeByUncheckedRows(field);
		} else {
			if(field.allInitialState){
				delete field.widget.currentRow[field.name + "_EXCEPT"];
			}
			(field.outData ? changeByOut : changeCurrentRow)(field, rows, field.name);
		}
		setChanged(field);


		if (field._afterSelected) {
			field._afterSelected(row);
		}

		if (field.change) field.change();
	}

	function afterClosePopup(field) {
		if (field.element) {
			field.element.focus();
		}
	}

	function isSelectFlag(columnName){
		return columnName === '__isSelected' || columnName === '-__isSelected';
	}

	function hasSelectFlag(orderColumns){
		return Util.isArray(orderColumns) ? orderColumns.some(isSelectFlag) : isSelectFlag(orderColumns);
	}

	function getOrderWithoutSelectedFlag(order){
		var orderColumns = order || [];
		orderColumns = Util.isArray(orderColumns) ? Util.clone(orderColumns) : [orderColumns];
		return orderColumns.filter(function(columnName){
			return !isSelectFlag(columnName);
		});
	}

	function afterLoadSelectDataSource(){
		setAllToggledInDataSource.apply(this);
	}

	function afterChangeOrder() {
		if(hasSelectFlag(this.order)){
			this.insertSelectFlag = true;
			this.order = getOrderWithoutSelectedFlag(this.order);
		} else {
			this.insertSelectFlag = false;
		}
	}

	function changeCurrentRow(field, rows, key, uncheckedRows){
		field.widget.currentRow[key] = [];
		if(field.dataSource.allToggled){
			var formattedSearchCriteria = [];
			if(field.selectWidget && field.selectWidget.searchCriteria){
				Object.keys(field.selectWidget.searchCriteria).forEach(function(key){
					if(field.selectWidget.searchCriteria[key]){
						formattedSearchCriteria.push({
							"name": key,
							"operator": "LIKE_ALL",
							"value": field.selectWidget.searchCriteria[key]
						});
					}
				});
			}
			var valueOption = {
				"__ALL": formattedSearchCriteria.concat(field.selectWidget.getDataSourceFilter() || [])
			};
			field.widget.currentRow[field.name] = valueOption;
		}
		if(uncheckedRows || !field.dataSource.allToggled) {
			if (Util.isArray(field.valueFields)) {
				rows.forEach(function(item) {
					field.widget.currentRow[key].push(field.valueFields.reduce(function(value, key) {
						value[key] = item[key];
						return value;
					}, {}));
				});
			} else {
				rows.forEach(function(item) {
					field.widget.currentRow[key].push(item[field.valueField]);
				});
			}
		}
	}

	function changeByUncheckedRows(field){
		field.dataSource.updateUncheckedRows();
		field.widget.currentRow[field.name] = [];
		var key = field.name + '_EXCEPT';
		changeCurrentRow(field, field.dataSource.uncheckedRows, key, true);
	}

	function changeByOut(field, rows, currentKey) {
		changeCurrentRow(field, rows, currentKey);
		self.setOutData(field, rows);
	}

	this.setOutData = function(field, rows) {
		var outData = field.outData || {};
		prepareCurrentRow(outData, field);
		var keys = Object.keys(outData);
		rows.forEach(function(row) {
			keys.forEach(function(key) {
				if (Array.isArray(outData[key])) {
					outData[key].forEach(function(valueKey) {
						field.widget.currentRow[valueKey].push(row[key]);
					});
				} else {
					field.widget.currentRow[outData[key]].push(row[key]);
				}
			});
		});
	};

	function prepareCurrentRow(outData, field) {
		var keys = Object.keys(outData);
		keys.forEach(function(key) {
			if (Array.isArray(outData[key])) {
				outData[key].forEach(function(valueKey) {
					field.widget.currentRow[valueKey] = [];
				});
			} else {
				field.widget.currentRow[outData[key]] = [];
			}
		});
	}

	$scope.maintainWindow = function(field) {
		if (field.maintainWindow) {
			var maintainWindowFilter = SelectService.buildMaintainFilter(field);
			ScreenService.openWindow(field.maintainWindow, maintainWindowFilter);
		}
	};

	$scope.$watchCollection('field.dataSourceFilter', propagateFieldDataSourceFilterToSelectWidget);
	function propagateFieldDataSourceFilterToSelectWidget(fieldDataSourceFilter) {
		if ($scope.field.selectWidget) {
			$scope.field.selectWidget.dataSourceFilter = fieldDataSourceFilter;
		}
	}

	this.hasSelectedRows = function hasSelectedRows(field) {
		var selectedRows = field.value();
		return selectedRows && selectedRows.length > 0;
	};
}